
"""
Basically, this should replace the runner functionality from trigger so that
we can more easily debug what's going on without sending stuff out
"""

import resultsdb_api

from . import config
from . import exceptions as exc


class BaseSender(object):
    def send(self, result):
        raise NotImplementedError

# these don't overlap so well but the best mapping seems to be SUCCESS == PASSED and anything else
# is FAILED. I don't see a place in here where INFO or NEEDS_INSPECTION are appropriate unless I'm
# misunderstanding what the outcomes from CI mean

CI_OUTCOMES = ('SUCCESS', 'FAILURE','ABORTED')
RESULTSDB_OUTCOMES = ('PASSED', 'INFO', 'FAILED', 'NEEDS_INSPECTION')

class ResultsdbSender(BaseSender):
    def __init__(self, rdb_connection=None):
        if rdb_connection is None:
            self.rdb_connection = resultsdb_api.ResultsDBapi(config.resultsdb_url)
        else:
            self.rdb_connection = rdb_connection

    def _check_testcase_exists(self, testcase):
        try:
            self.rdb_connection.get_testcase(testcase)
        except resultsdb_api.ResultsDBapiException as e:
            raise exc.SenderError("Testcase was not found")

    def _create_testcase(self, testcase):
        self.rdb_connection.create_testcase(testcase)

    def _translate_outcome(self, outcome):
        if outcome == 'SUCCESS':
            return 'PASSED'
        else:
            return 'FAILED'


    # topic is the testcase
    # add all data as kv pairs
    def send(self, result):
        testcase_name = result['topic']

        # make sure that the testcase exists, if not, create it
        try:
            self._check_testcase_exists(testcase_name)
        except exc.SenderError as e:
            self._create_testcase(testcase_name)

        # translate the outcome from CI to resultsdb-speak
        rdb_outcome = self._translate_outcome(result['status'])

        # construct the data that is to be sent
        ref_url = result['build_url']

        # send the result to resultsdb
        self.rdb_connection.create_result(
            outcome=rdb_outcome,
            testcase=testcase_name,
            group=None,
            note=None,
            ref_url=ref_url,
            **result)


class StreamSender(BaseSender):
    def send(self, result):
        print(result)


_senders = {"ResultsdbSender": ResultsdbSender, "StreamSender": StreamSender}


def get_sender(sender_type):
    return _senders[sender_type]
